=== modified file 'plugin/csv/ha_tina.cc'
--- plugin/csv/ha_tina.cc	2010-06-08 06:42:22 +0000
+++ plugin/csv/ha_tina.cc	2010-06-10 05:33:28 +0000
@@ -53,6 +53,8 @@
 
 #include <fcntl.h>
 
+#include <algorithm>
+#include <vector>
 #include <string>
 #include <map>
 
@@ -88,19 +90,6 @@
  *****************************************************************************/
 
 /*
-  Used for sorting chains with qsort().
-*/
-static int sort_set (tina_set *a, tina_set *b)
-{
-  /*
-    We assume that intervals do not intersect. So, it is enought to compare
-    any two points. Here we take start of intervals for comparison.
-  */
-  return ( a->begin > b->begin ? 1 : ( a->begin < b->begin ? -1 : 0 ) );
-}
-
-
-/*
   If frm_error() is called in table.cc this is called to find out what file
   extensions exist for this Cursor.
 */
@@ -551,12 +540,10 @@
     They are not probably completely right.
   */
   current_position(0), next_position(0), local_saved_data_file_length(0),
-  file_buff(0), chain_alloced(0), chain_size(DEFAULT_CHAIN_LENGTH),
-  local_data_file_version(0), records_is_known(0)
+  file_buff(0), local_data_file_version(0), records_is_known(0)
 {
   /* Set our original buffers from pre-allocated memory */
   buffer.set((char*)byte_buffer, IO_SIZE, &my_charset_bin);
-  chain= chain_buffer;
   file_buff= new Transparent_file();
 }
 
@@ -661,36 +648,10 @@
 */
 int ha_tina::chain_append()
 {
-  if ( chain_ptr != chain && (chain_ptr -1)->end == current_position)
-    (chain_ptr -1)->end= next_position;
+  if (chain.size() > 0 && chain.back().second == current_position)
+    chain.back().second= next_position;
   else
-  {
-    /* We set up for the next position */
-    if ((off_t)(chain_ptr - chain) == (chain_size -1))
-    {
-      off_t location= chain_ptr - chain;
-      chain_size += DEFAULT_CHAIN_LENGTH;
-      if (chain_alloced)
-      {
-        if ((chain= (tina_set *) realloc(chain, chain_size)) == NULL)
-          return -1;
-      }
-      else
-      {
-        tina_set *ptr= (tina_set *) malloc(chain_size * sizeof(tina_set));
-        if (ptr == NULL)
-          return -1;
-        memcpy(ptr, chain, DEFAULT_CHAIN_LENGTH * sizeof(tina_set));
-        chain= ptr;
-        chain_alloced++;
-      }
-      chain_ptr= chain + location;
-    }
-    chain_ptr->begin= current_position;
-    chain_ptr->end= next_position;
-    chain_ptr++;
-  }
-
+    chain.push_back(make_pair(current_position, next_position));
   return 0;
 }
 
@@ -1122,7 +1083,7 @@
   current_position= next_position= 0;
   stats.records= 0;
   records_is_known= 0;
-  chain_ptr= chain;
+  chain.clear();
 
   blobroot.init_alloc_root(BLOB_MEMROOT_ALLOC_SIZE);
 
@@ -1211,14 +1172,14 @@
   to the given "hole", stored in the buffer. "Valid" here means,
   not listed in the chain of deleted records ("holes").
 */
-bool ha_tina::get_write_pos(off_t *end_pos, tina_set *closest_hole)
+bool ha_tina::get_write_pos(off_t *end_pos, vector< pair<off_t, off_t> >::iterator &closest_hole)
 {
-  if (closest_hole == chain_ptr) /* no more chains */
+  if (closest_hole == chain.end()) /* no more chains */
     *end_pos= file_buff->end();
   else
     *end_pos= std::min(file_buff->end(),
-                       closest_hole->begin);
-  return (closest_hole != chain_ptr) && (*end_pos == closest_hole->begin);
+                       closest_hole->first);
+  return (closest_hole != chain.end()) && (*end_pos == closest_hole->first);
 }
 
 
@@ -1236,9 +1197,9 @@
   blobroot.free_root(MYF(0));
   records_is_known= 1;
 
-  if ((chain_ptr - chain)  > 0)
+  if (chain.size() > 0)
   {
-    tina_set *ptr= chain;
+    vector< pair<off_t, off_t> >::iterator ptr= chain.begin();
 
     /*
       Re-read the beginning of a file (as the buffer should point to the
@@ -1250,8 +1211,7 @@
       The sort is needed when there were updates/deletes with random orders.
       It sorts so that we move the firts blocks to the beginning.
     */
-    internal::my_qsort(chain, (size_t)(chain_ptr - chain), sizeof(tina_set),
-                       (qsort_cmp)sort_set);
+    sort(chain.begin(), chain.end());
 
     off_t write_begin= 0, write_end;
 
@@ -1282,10 +1242,10 @@
       if (in_hole)
       {
         /* skip hole */
-        while (file_buff->end() <= ptr->end && file_buffer_start != -1)
+        while (file_buff->end() <= ptr->second && file_buffer_start != -1)
           file_buffer_start= file_buff->read_next();
-        write_begin= ptr->end;
-        ptr++;
+        write_begin= ptr->second;
+        ++ptr;
       }
       else
         write_begin= write_end;

=== modified file 'plugin/csv/ha_tina.h'
--- plugin/csv/ha_tina.h	2010-04-23 17:43:11 +0000
+++ plugin/csv/ha_tina.h	2010-06-10 05:33:28 +0000
@@ -23,7 +23,6 @@
 #include <sys/stat.h>
 #include "transparent_file.h"
 
-#define DEFAULT_CHAIN_LENGTH 512
 /*
   Version for file format.
   1 - Initial Version. That is, the version when the metafile was introduced.
@@ -60,11 +59,6 @@
   uint32_t data_file_version;   /* Version of the data file used */
 };
 
-struct tina_set {
-  off_t begin;
-  off_t end;
-};
-
 class ha_tina: public drizzled::Cursor
 {
   drizzled::THR_LOCK_DATA lock;      /* MySQL lock */
@@ -83,16 +77,13 @@
     deletes/updates. It is used in doEndTableScan() to get rid of them
     in the end of the query.
   */
-  tina_set chain_buffer[DEFAULT_CHAIN_LENGTH];
-  tina_set *chain;
-  tina_set *chain_ptr;
-  unsigned char chain_alloced;
-  uint32_t chain_size;
+  std::vector< std::pair<off_t, off_t> > chain;
   uint32_t local_data_file_version;  /* Saved version of the data file used */
   bool records_is_known;
   drizzled::memory::Root blobroot;
 
-  bool get_write_pos(off_t *end_pos, tina_set *closest_hole);
+  bool get_write_pos(off_t *end_pos,
+                     std::vector< std::pair<off_t, off_t> >::iterator &closest_hole);
   int open_update_temp_file_if_needed();
   int init_tina_writer();
   int init_data_file();
@@ -101,8 +92,6 @@
   ha_tina(drizzled::plugin::StorageEngine &engine, drizzled::TableShare &table_arg);
   ~ha_tina()
   {
-    if (chain_alloced)
-      free(chain);
     if (file_buff)
       delete file_buff;
   }

